@@ -5,9 +5,6 @@ use Test::Assert ':all';
 use Test::More import => ['!fail'];
 use Test::Differences;
 
-Test::Builder->new->no_plan;
-Test::Builder->new->no_ending(1);
-
 my @a = (1,2,3,4,5);
 my @b = (1,2,2,4,5);
 
